// Function for the dynamics of the problem
// y_dot = dynamics(z) = dynamics(y,u,z,p)

// Input :
// t : time
// dim_* is the dimension of next vector in the declaration
// state : vector of state variables
// control : vector of control variables
// algebraicvars : vector of algebraic variables
// optimvars : vector of optimization variables
// constants : vector of constants

// Output :
// state_dynamics : vector giving the expression of the dynamic of each state variable.

#include "header_dynamics"
{
   int i;

   double h = 1.0/(dim_state-1);
   double ah = constants[0]/pow(h,2);
   double gamma = constants[3];
   double delta = constants[4];

	 Tdouble normstate = 0.0;


/*
   // DIRICHLET

   state_dynamics[0] = ah*(- 2*state[0] + state[1]);	
   state_dynamics[dim_state-2] = ah*(- 2*state[dim_state-2] + y[dim_state-3]);

   for(i=1;i<dim_state-2;i++)
   {
       state_dynamics[i] = ah*(state[i-1] - 2*state[i] + state[i+1]);
   }

   for(i=0;i<(int)constants[2];i++)
   {
       state_dynamics[i] += constants[1]*control[0];
   }
*/

  // NEUMANN    

   state_dynamics[0] = ah*(- state[0] + state[1] + control[0]*h);	
   state_dynamics[dim_state-2] = ah*(- state[dim_state-2] + state[dim_state-3]);

   for(i=1;i<dim_state-2;i++)
   {
       state_dynamics[i] = ah*(state[i-1] - 2*state[i] + state[i+1]);
   }



   // CRITERION Min int (state^2 + gamma control + delta control^2)
   for(i=0;i<dim_state-2;i++)
   {
       normstate = normstate + pow(state[i],2);
   }
   state_dynamics[dim_state-1] = normstate / 2.0 + gamma*control[0] + delta*pow(control[0],2);

}