// Function for the dynamics of the problem
// y_dot = dynamics(z) = dynamics(y,u,z,p)

// Input :
// t : time
// dim_* is the dimension of next vector in the declaration
// state : vector of state variables
// control : vector of control variables
// algebraicvars : vector of algebraic variables
// optimvars : vector of optimization variables
// constants : vector of constants

// Output :
// state_dynamics : vector giving the expression of the dynamic of each state variable.

//#include <cmath>
#include "header_dynamics"
{
	int i;
	int ia;
	int ib;

	int na = 2;     // ages 1:young, 2:old
	int nb = 14;	// illness duration
	int no = 6;		// incubation phase
  
    // "state" contains in this order : y(na), yb(na), z(na*nb), h(na*nb), pointers:
	int pz = 4;                  //  address of z in vector "state"
    int ph = pz + na * nb;   	 //  address of h in vector "state"
    int pD = ph + na * nb;		 // address of death toll D1 and D2 in the vector "state"

    // constants vector: delta, eta, nu, gamma, C, penalties, eps-sqrt, c_e, pointers:
    int pdelta = 0;   //  address of delta in vector "constants"
    int peta = 2;     //  address of eta in vector "constants"
    int pnu = 4;      //  address of nu in vector "constants"
    int pgamma = 32;  //  address of gamma in vector "constants"
    int pC = 60;      //  address of C (hospital capacity) in vector "constants"
   
    // Tdouble type for "varying double"
    //  Useful for automatic differentiation
    //  zz = Z, sum of z variables for n0<b=<nb and all a.
    Tdouble zz = 0.0;
    for(i = na * (no - 1); i < na * nb; i++){
		zz += state[pz+i];
    }
   
    // Dynamics of susceptible states
    state_dynamics[0] = - constants[pdelta] * (1 - control[0]) * zz * state[0];
    state_dynamics[1] = - constants[pdelta + 1] * (1 - control[1]) * zz * state[1];

    // Dynamics of immune states
    //   When ia and ib start from zero, then z(ia,ib) at address pz +ia+na*ib
    state_dynamics[2] = state[pz + na * (nb - 1)] + state[ph + na * (nb - 1)];
    state_dynamics[3] = state[pz + 1 + na * (nb - 1)] + state[ph + 1 + na * (nb - 1)];

    // Dynamics of z variables
    for(ia = 0 ; ia < na; ia++){
		// Case when ib=0
        state_dynamics[pz + ia] = -state[pz + ia] - state_dynamics[ia] ;
        
        for(ib = 1; ib < nb; ib++){
            // In math notation:    z^{k+1}_j = z^k_j + z^k_{j-1} - z^k_j 
            state_dynamics[pz+ia+na*ib] = -state[pz + ia + na*ib] 
										  + (1 - constants[pnu + ia + na * (ib - 1)]) * state[pz + ia + na * (ib - 1)] ; 
        }
     }
   
     // h variables
     // As for z, h(ia,ib) address ph+ia+na*ib
     // hh: sum of variables h over a,b
     Tdouble hh = 0;
     for( ia = 0; ia < na * nb; ia++){
		hh += state[ph + ia];
     }
    
     // dif_cap: difference between hh and the total hospital capacity C
     Tdouble dif_cap = hh - constants[pC];
    
     // eps: epsilon is involved in the approximation of the positive part of (hh-C)
     double eps_sqrt = constants[64];

     for(ia = 0; ia < na; ia++){ 
		// case ib=0: zero inital condition
        state_dynamics[ph + ia] = - state[ph + ia] ; 

        for(ib=1; ib<nb; ib++){
			// h^{k+1}_j = h^k_j + h^k_{j-1} - h^k_j 
			state_dynamics[ph + ia + na * ib] = -state[ph + ia +na * ib]
                                                + constants[pnu + ia + na * (ib - 1)] * state[pz + ia + na * (ib - 1)] 
											    + (1 - constants[peta + ia]) * state[ph + ia + na * (ib - 1)]
                                                - constants[pgamma + ia + na * (ib - 1)] * 0.5 * (dif_cap + sqrt(dif_cap * dif_cap + eps_sqrt * eps_sqrt)) * state[ph + ia + na * (ib - 1)] / (hh + constants[pC]);
        }
     }
   
	//Death toll D1: cumulative number of deaths for age 1 
	Tdouble death1 = 0.0;
	for(ib = 1; ib < nb; ib++){
			ia = 0;
	        death1 +=  constants[peta + ia] * state[ph + ia + na * (ib -  1)]
						+ constants[pgamma + ia + na * (ib - 1)] * 0.5 * (dif_cap + sqrt(dif_cap * dif_cap + eps_sqrt * eps_sqrt)) * state[ph + ia + na * (ib - 1)] / ( hh + constants[pC]);
	}            
	// D1^k+1 = D1^k+death1
	state_dynamics[pD] = death1;
    
    //Death toll D2: cumulative number of deaths for age 1
    Tdouble death2 = 0.0;
	for(ib = 1; ib < nb; ib++){
			ia = 1;
	        death2 +=  constants[peta + ia] * state[ph + ia + na * (ib -  1)]
				       + constants[pgamma + ia + na * (ib - 1)] * 0.5 * (dif_cap + sqrt(dif_cap * dif_cap + eps_sqrt * eps_sqrt)) * state[ph + ia + na * (ib - 1)] / ( hh + constants[pC]);
	}
    // D2^k+1 = D2^k+death2
	state_dynamics[pD + 1] = death2;
	
	
	//Integral of controls 
	state_dynamics[62] = control[0];
	state_dynamics[63] = control[1];
	 

    // CRITERION : running cost u(t)
    state_dynamics[dim_state-1] = constants[65] * control[0] + constants[66]  * control[1] ;

}
