// Function for the path constraints of the problem
// a <= g(t,y,u,z,p) <= b

// Input :
// dim_path_constraints : number of path constraints
// t : time
// dim_* is the dimension of next vector in the declaration
// state : vector of state variables
// control : vector of control variables
// algebraicvars : vector of algebraic variables
// optimvars : vector of optimization variables
// constants : vector of constants

// Output :
// path_constraints : vector of path constraints expressions ("g" in the example above)

#include "header_pathcond"
{
	int ia;
    int ib;

	// constants: pointers
    int pdelta = 0;   //  address of delta in vector "constants"
    int peta = 2;     //  address of eta in vector "constants"
    int pnu = 4;      //  address of nu in vector "constants"
    int pgamma = 32;  //  address of gamma in vector "constants"
    int pC = 60;      //  address of C (hospital capacity) in vector "constants"
   
    int na = 2;     // ages 1:young, 2:old
	int nb = 14;	// illness duration
  
    //  address of h in vector "state"
	int ph = 4 + na*nb;   

    // Population in need of hospitalization
    Tdouble hh = 0.0; 
	for( ia = 0; ia < na * nb; ia++){
		hh += state[ph + ia];
    }

    //  constraint: hospitalization capacity minus population in hospitals
    // optimvars[0] minimize the maximum hh over the time.
   	path_constraints[0] =  hh - optimvars[0]; 
	
	// dif_cap: difference between hh and the total hospital capacity C
    Tdouble dif_cap = hh - constants[pC];
	
	// eps: epsilon is involved in the approximation of the positive part of (hh-C)
    double eps_sqrt = constants[64];	
		
	// death toll due to the illness for y_1 and y_2
	Tdouble death1s = 0.0;
	for(ib = 1; ib < nb; ib++){
			ia = 0;
	        death1s +=  constants[peta + ia] * state[ph + ia + na * (ib -  1)];
    }              
 
    Tdouble death1w = 0.0;
	for(ib = 1; ib < nb; ib++){
			ia = 1;
	        death1w +=  constants[peta + ia] * state[ph + ia + na * (ib -  1)];
    }              
     
    // death toll due to saturation of hospital capacity for y_1 and y_2
    Tdouble death2s = 0.0;
	for(ib = 1; ib < nb; ib++){
			ia = 0;
			death2s +=  constants[pgamma + ia + na * (ib - 1)] * 0.5 * (dif_cap + sqrt(dif_cap * dif_cap + eps_sqrt * eps_sqrt)) * state[ph + ia + na * (ib - 1)] / ( hh + constants[pC]);
	}
     
    Tdouble death2w = 0.0;
	for(ib = 1; ib < nb; ib++){
			ia = 1;
			death2w +=  constants[pgamma + ia + na * (ib - 1)] * 0.5 * (dif_cap + sqrt(dif_cap * dif_cap + eps_sqrt * eps_sqrt)) * state[ph + ia + na * (ib - 1)] / ( hh + constants[pC]);
	}
  
	path_constraints[1] = death1s;
	path_constraints[2] = death1w;
	path_constraints[3] = death2s;
	path_constraints[4] = death2w;
}
// EOF
